import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { map, first, mergeMap } from 'rxjs/operators';


import { Movie } from './movie';
import { MoviesComponent } from './movies/movies.component';

interface IMovieData {results: Movie[]; }
export interface IVideoData {results: IVideo[]; }
export interface IVideo {'id': string; 'key': string; 'name': string; 'site': string; 'type': string; }

@Injectable()
export class HttpService {

  private movies: Movie[]|undefined;
  private key : string | undefined;

  constructor(private http: HttpClient) { }

  public getMovies(): Observable<Movie[]> {
    return this.http.get<Movie[]>('http://api.themoviedb.org/3/search/movie?query=skyfall&api_key=81c50c197b83129dd4fc387ca6c8c323');
  }

  public getMovie(title: string): Observable<Movie[]> {
// tslint:disable-next-line: max-line-length
    return this.http.get<IMovieData>('http://api.themoviedb.org/3/search/movie?query=' + title + '&api_key=81c50c197b83129dd4fc387ca6c8c323')
    .pipe(map(movies => {
       return movies.results.filter(movie => movie.poster_path !== null).map(movie => {
        return { id: movie.id,
                 title: movie.title,
                 poster_path: 'https://image.tmdb.org/t/p/w185_and_h278_bestv2' + movie.poster_path,
                 adult: movie.adult,
                 overview: movie.overview,
                 release_date: movie.release_date,
                 genres: movie.genres,
                 vote_average: movie.vote_average } as Movie; });

    }));
  }


  public getMostPopularMovies(): Observable<Movie[]> {
    // tslint:disable-next-line: max-line-length
        return this.http.get<IMovieData>('http://api.themoviedb.org/3/discover/movie?sort_by=popularity.desc&api_key=81c50c197b83129dd4fc387ca6c8c323')
        .pipe(map(movies => {
            return movies.results.filter(movie => movie.poster_path !== null).map(movie => {
            return { id : movie.id,
                     title : movie.title,
                     poster_path : 'https://image.tmdb.org/t/p/w185_and_h278_bestv2' + movie.poster_path,
                     adult : movie.adult,
                     overview : movie.overview,
                     release_date : movie.release_date,
                     genres : movie.genres,
                     vote_average : movie.vote_average } as Movie; });

        }));
      }

public getHigestRatedMovies(): Observable<Movie[]> {
// tslint:disable-next-line: max-line-length
    return this.http.get<IMovieData>('http://api.themoviedb.org/3/discover/movie?certification_country=US&certification=R&primary_release_year=2022&sort_by=vote_average.desc&api_key=81c50c197b83129dd4fc387ca6c8c323')
    .pipe(map(movies => {
      return movies.results.filter(movie => movie.poster_path !== null).map(movie => {
        return { id : movie.id,
                 title : movie.title,
                 poster_path : 'https://image.tmdb.org/t/p/w185_and_h278_bestv2' + movie.poster_path,
                 adult : movie.adult,
                 overview : movie.overview,
                 release_date : movie.release_date,
                 genres : movie.genres,
                 vote_average : movie.vote_average } as Movie; });

    }));
  }
public getUpcommingMovies(): Observable<Movie[]> {
    return this.http.get<IMovieData>('http://api.themoviedb.org/3/movie/upcoming?api_key=81c50c197b83129dd4fc387ca6c8c323')
    .pipe(map(movies => {
       return movies.results.filter(movie => movie.poster_path !== null).map(movie => {
         return { id : movie.id,
                  title : movie.title,
                  poster_path : 'https://image.tmdb.org/t/p/w185_and_h278_bestv2' + movie.poster_path,
                  adult : movie.adult,
                  overview : movie.overview,
                  release_date : movie.release_date,
                  genres : movie.genres,
                  vote_average : movie.vote_average } as Movie; });

    }));
  }

  public getVideo(id: number): Observable<IVideo> {
     // tslint:disable-next-line: max-line-length
     const video = this.http.get<IVideoData>('https://api.themoviedb.org/3/movie/' + id + '/videos?api_key=81c50c197b83129dd4fc387ca6c8c323')
    .pipe(
      map(videos => videos.results),
      mergeMap(processArray => processArray.filter(x => x.key !== null)),
      first());
     return video;
}

/*public getVideo(id: number): Observable<IVideo> {
    console.log('getvideo1: ' + id);
    const video = this.http.get<IVideoData>('https://api.themoviedb.org/3/movie/' + id + '/videos?api_key=81c50c197b83129dd4fc387ca6c8c323')
    .pipe(map(videos => {
       return videos.results; })).pipe(mergeMap(processArray => {
        return processArray.filter(x => x.key !== null);
      })).pipe(first());
    console.log('video: ' + video);
    return video;
}
*/

public async getVideoPath(id: number): Promise<string> {
    console.log('getvideo1: ' + id);
// tslint:disable-next-line: max-line-length
    const videos = await this.http.get<IVideoData>('https://api.themoviedb.org/3/movie/' + id + '/videos?api_key=81c50c197b83129dd4fc387ca6c8c323').toPromise();
    if (videos){
      return videos.results[0].key;
    }
    return "";
}

//public getYouTubePath(id: number): string {
//      const path: string = 'https://www.youtube.com/embed/' + JSON.stringify(this.getVideoPath(id));
//      return path;
//}
}
